/*******************************************************************************
 *******************************************************************************
 **                                                                           **
 **                                                                           **
 **  Copyright 2003-2006 Tom Bascom, Greenfield Technologies                  **
 **  http://www.greenfieldtech.com                                            **
 **                                                                           **
 **  ProTop is free software; you can redistribute it and/or modify it        **
 **  under the terms of the GNU General Public License (GPL) as published     **
 **  by the Free Software Foundation; either version 2 of the License, or     **
 **  at your option) any later version.                                       **
 **                                                                           **
 **  ProTop is distributed in the hope that it will be useful, but WITHOUT    **
 **  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or    **
 **  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License     **
 **  for more details.                                                        **
 **                                                                           **
 **  See TERMS.TXT for more information regarding the Terms and Conditions    **
 **  of use and alternative licensing options for this software.              **
 **                                                                           **
 **  A copy of the GPL is in GPL.TXT which was provided with this package.    **
 **                                                                           **
 **  See http://www.fsf.org for more information about the GPL.               **
 **                                                                           **
 **                                                                           **
 *******************************************************************************
 *******************************************************************************
 *
 * protoplib.p
 *
 * Library of common infrastructure functions for the ProTop family of programs
 *
 *
 * Known Bugs & Issues:
 *
 *
 * To Do:
 *
 *
 * Author:
 *
 *	Tom Bascom, Greenfield Technologies
 *	http://www.greenfieldtech.com
 *	August 28, 2003
 *
 *
 * History:
 *
 *	Accepted changes from Sam Paakki regarding "quit" function and PROPATH
 *	September 26, 2003
 *
 *      Accepted changes from Patrick Tingen to set html output dir,   
 *      eliminating the nasty curr-page shared variable, simplifying the
 *	release# and adding protop-url 
 *      October 30, 2003
 * 
 */

{lib/protop.i}

define new global shared temp-table tt_process no-undo
  field pid     as character
  field cmdLine as character
  index pid-idx is unique primary pid
.

run lib/windows.p.							/* load lib/windows32.p or lib/windows64.p as appropriate	*/

subscribe to "setLogName" anywhere run-procedure "setLogName".
subscribe to "logMsg"     anywhere run-procedure "doLogMsg".


/*** Install self as a session super-procedure
 ***
 ***/

session:add-super-procedure( this-procedure ).

return.


/** Common functions
 **
 **/


/* Thanks to Wim van der Ham -- the very first community contribution!
 *
 * Now directly supported in 4gl but I keep it around for the memories :)
 *
 * function LOGICAL returns logical ( input p_text as character ):
 *   return ( p_text = "true" or p_text = "yes" ).
 * end.
 *
 */


/* return a corrected decimal on the assumption that the sign bit should not have been flipped
 *
 */

function unsignMe returns decimal ( input s as decimal, i as integer ):

  if s < 0 then s = s + exp( 2, i ).

  return s.

end.


/* Variations on the hit ratio theme -- Gus' method:
 *
 *	http://www.peg.com/lists/dba/history/200209/msg00057.html
 *
 */

define variable warned as logical no-undo.	/*** debugging code ***/

function hr returns decimal ( input lr as decimal, input osr as decimal, output hr-str as character, output hr as decimal, output mr as decimal ):

  define variable lrx  as decimal no-undo.
  define variable osrx as decimal no-undo.

  define variable hr-top as decimal no-undo.
  define variable hr-bot as decimal no-undo.

  assign
    lrx       = lr
    osrx      = osr
    hr-top    = lrx / osrx
    hr-bot    = 1.0
    hr        = 100.0 * (( lrx - osrx ) / lrx )
    mr        = 100.0 - hr
  .

  if hr < 0 and warned = no then
    do:

	hr = 0.
	warned = yes.

	/** Debugging -- this is very bad!!!
	 **

        display
	   skip(1)
           "  BOGUS HIT RATIO Calculation!!!  " skip
	   skip(1)
           "       lr:" lr     skip
           "      osr:" osr    skip
           "      lrx:" lrx    skip
           "     osrx:" osrx   skip
           "   hr-top:" hr-top skip
           "   hr-bot:" hr-bot skip
           "       hr:" hr     skip
           "       mr:" mr    skip
           skip(1)
           "   Please send this data to:" skip
           "       tom@greenfieldtech.com" skip
           skip(1)
         with no-labels.

	 **
	 **/

    end.

  if hr < 0 then
    do:
      hr = 0.
      hr-str = "".
      return hr.
    end.

  if hr < 1 then		  /* deal with pathologically bad ratios...	*/
    assign
      hr-bot = 1.0 / hr-top
      hr-top = 1.0
    .

  hr-str = string( truncate( hr-top, 0 )) + ":" + string( truncate( hr-bot, 0 )).
  if hr-str = ? then hr-str = "0:0".
  if length( hr-str ) < 10 then hr-str = fill( " ", 10 - length( hr-str )) + hr-str.

  return hr.

end.


function do-SumSample returns logical ( output p_index as integer, output p_time  as integer ):

  define variable r as character no-undo case-sensitive initial "r".
  define variable s as character no-undo case-sensitive initial "s".

  publish "get-RateRaw" ( output r ).		/*** I'm missing something...	***/
  publish "get-SumSample" ( output s ).

  if s = "S" then
    assign
      p_index = 4
      p_time  = xtime.
   else
    assign
      p_index = 5
      p_time  = itime.

  return true.

end.	/* do-SumSample */


/* useful when there is no db conection and thus no _myconnection
 */

procedure GetCurrentProcessId external "kernel32":
  define return parameter pid as long.
end.


function myPID returns character ():

  define variable pid as integer no-undo initial ?.

  if opsys begins "WIN" then
    do:
      run GetCurrentProcessId( output pid ).
    end.
   else		/* calling the libc getpid function would be nice but finding libc.so is a pain... */
    do:
      input stream inStrm through value( "echo $PPID" ).
      import stream inStrm pid.
      input stream inStrm close.
    end.

  return string( pid ).

end.


function getHostName returns character ():

  define variable hostName as character no-undo.

  if ldbName( "DICTDB" ) <> ? then			/* if it is a client server connection we want the *db* host	*/
    do:

      hostName = dbParam( ldbName( "DICTDB" )).

      if index( hostName, "-H " ) <= 0 then
        hostName = "".
       else
        hostName = entry( 2, entry( 1, substring( hostName, index( hostName, "-H " ))), " " ).

    end.

  if hostName = ? or hostName = "" then			/* if there was no -H involved just use uname -n		*/
    do:

      if opsys = "unix" then
        do:
          input stream inStrm through value( "uname -n" ).
          import stream inStrm hostName.
          input stream inStrm close.
        end.
       else
        do:
          input stream inStrm through value( "hostname" ).			/* more reliable for Windows		*/
          import stream inStrm hostName.
          input stream inStrm close.
        end.

      if ( hostName = ? or hostName = "" ) and opsys begins "win" then		/* should never actually happen		*/
        do:
          input stream inStrm through value( 'wmic os get csname' ).
          import stream inStrm unformatted ^.
          import stream inStrm unformatted hostName.
          input stream inStrm close.
        end.

      if hostName = ? or hostName = "" then hostName = "Unknown".

    end.

  return hostName.

end.


/* myCustId
 *
 */

procedure myCustId:

  define output parameter custId as character no-undo.

  custId = "".

  file-info:file-name = "etc/custid.cfg".
  if file-info:full-pathname <> ? then
    do:
      input stream inStrm from value( file-info:full-pathname ).
      import stream inStrm unformatted custId.
      input stream inStrm close.
    end.

  return.

end.


/* chkDbgMode
 *
 * check for changes to the debug level
 *
 * i.e.
 *
 *	echo 5 > $TMPDIR/monName.friendlyName.dbg
 *
 */

define variable dbgChgDate   as date        no-undo.
define variable dbgChgTime   as integer     no-undo.

procedure chkDbgMode:

  define input        parameter dbgFileName as character no-undo.
  define input-output parameter dbgMode     as integer   no-undo.

  file-info:file-name = dbgFileName.
  if file-info:full-pathname <> ? and ( file-info:file-mod-date <> dbgChgDate or file-info:file-mod-time <> dbgChgTime ) then
    do:
      assign								/* remember when we last looked!			*/
        dbgChgDate = file-info:file-mod-date
        dbgChgTime = file-info:file-mod-time
      .
      input stream inStrm from value( file-info:full-pathname ).
      import stream inStrm dbgMode.
      input stream inStrm close.
      message now "dbgMode:" dbgMode.
    end.

  return.

end.


/* chkProfiler
 *
 * check for requests to start or stop profiling
 *
 * i.e.
 *
 *	echo 5 > $TMPDIR/monName.friendlyName.dbg
 *
 */

define variable prfChgDate   as date        no-undo.
define variable prfChgTime   as integer     no-undo.

procedure chkProfiler:

  define input        parameter prfFileName as character no-undo.
  define input-output parameter prfMode as character no-undo.

  if prfFileName = ? or prfFileName = "" then
    return.

  file-info:file-name = prfFileName.

  if file-info:full-pathname = ? and prfChgDate = ? then		/* no flag file exists and none was being tracked	*/
    return.

  if file-info:full-pathname = ? and prfChgDate <> ? then		/* a flag that was being tracked has disappeared	*/
    do:
      assign
        prfChgDate = ?
        prfMode    = "off"
      .
      run lib/zprof_off.p.
    end.

  /* a flag exists and the timestamp has changed!
   */

  if file-info:full-pathname <> ? and ( file-info:file-mod-date <> prfChgDate or file-info:file-mod-time <> prfChgTime ) then
    do:

      assign								/* remember when we last looked!			*/
        prfChgDate = file-info:file-mod-date
        prfChgTime = file-info:file-mod-time
      .

      input stream inStrm from value( file-info:full-pathname ).
      import stream inStrm prfMode.
      input stream inStrm close.

      if prfMode = "on" or  prfMode begins "y" then			/* enable profiling					*/
        do:
          prfMode = "on".
          run lib/zprof_on.p ( "", "", no ).				/* a GUID will be used for the file name		*/
        end.
       else								/* anything but "on" or "y*" = off			*/
        do:
          run lib/zprof_off.p.
        end.

    end.

  if dbgMode > 3 then
    message now "prfMode:" prfMode.

  return.

end.


/* monName
 *
 */

procedure getMonName:

  define input-output parameter monName as character.

  define variable i as integer no-undo.

  do i = 1 to num-entries( session:startup-parameters ):
    if entry( i, session:startup-parameters ) begins "-p " then
      do:
        monName = substring( entry( i, session:startup-parameters ), 4 ).		/* remove the "-p "			*/
        leave.
      end.
  end.

  monName = substring( monName, r-index( monName, "/" ) + 1 ).		/* take the part from the last "/" onward		*/
  monName = substring( monName, r-index( monName, "~\" ) + 1 ).		/* or "~\" if we are on Windows				*/

  if r-index( monName, "." ) = length( monName ) - 1 then
    monName = substring( monName, 1, length( monName ) - 2 ).		/* get rid of the ".p" (or .r)				*/

  return.

end.


procedure mkFlag:

  define input parameter flgFileName as character no-undo.

  define variable osPID     as character no-undo.			/* process id from "ps"					*/
  define variable myPID     as character no-undo.			/* the current process' PID				*/

  define variable flgPID    as character no-undo.			/* process id in .flg file				*/
  define variable flgDate   as character no-undo.			/* the flag file date					*/
  define variable flgTime   as character no-undo.			/* the flag file time					*/

  define variable monName   as character no-undo.			/* monitor name						*/
  define variable cmdLine   as character no-undo.			/* command line of process with flgPID			*/

  myPID = myPID().

  if dbgMode >= 5 then message now "mkFlag: myPID =" myPID " flagName =" flgFileName.

  file-info:file-name = flgFileName.
  if file-info:full-pathname <> ? then					/* the flag file exists -- but it might be stale	*/
    do:

      input stream inStrm from value( file-info:full-pathname ).	/* what PID created it?					*/
      import stream inStrm flgDate flgTime flgPID.			/* date time PID					*/
      input stream inStrm close.

      flgPID = trim( flgPID ).

      if dbgMode >= 5 then message now "mkFlag: flag file" file-info:full-pathname "exists. flgPID =" flgPID flgDate string( flgTime, "hh:mm:ss" ).

      if myPID = flgPID then						/* if the PID in the flag file = my PID...		*/
        do:
          message now "mkFlag: WTF? myPID = flgPID?" flgDate flgTime flgPID "this should not be possible".
          return.
        end.

      /* if the PID in the flag file <> my PID, I didn't create it, so who did?
       */

      if dbgMode >= 2 then message now "mkFlag: flag file exists.  Checking for stale flag file...".

      /* check if the indicated PID is still running and if it is an _progres process with the proper -p procedure
       */

      if opsys = "unix" then
        monName = "util/" + entry( 1, substring( flgFileName, r-index( flgFileName, "/" ) + 1 ), "." ).
       else
        monName = "util~\" + entry( 1, substring( flgFileName, r-index( flgFileName, "~\" ) + 1 ), "." ).

      if dbgMode >= 5 then
        message now substitute( "looking for _progres process &1, running &2", flgPID, monName ).

      cmdLine = "".
      run getAgentByPID( flgPID, output cmdLine ).

      if dbgMode >= 5 then
        message now substitute( "flgPID: &1, cmdLine: &2", flgPID, cmdLine ).

      if index( cmdLine, substitute( " -p &1", monName )) > 0 then	/* a process WAS found and it isn't us!			*/
        do:								/* (we already checked myPID above)			*/
          if dbgMode >= 2 then
            do:
              message now "Flag file:" flgFileName "exists and is owned by PID:" flgPID "cmdLine:" cmdLine.
              message now "It would appear that there is already a monitor running.".
              message now "Cowardly refusing to start another.".
            end.
          quit.								/* this instance does not need to start, so quit	*/
        end.
       else								/* no appropriate process with that PID is running	*/
        do:
          message now "Flag file:" flgFileName "exists but appears to be stale.  PID:" flgPID "is not running." osPID.
          message now "Deleting" flgFileName.
          os-delete value( flgFileName ).				/* just in case this is a mistake -- give the other guy	*/
          pause 30 no-message.						/* a chance to notice and exit...			*/
        end.					

    end.

  /* the flag file did not exist or it was stale and got deleted, so create a new one and carry on.
   */

  if myPID = ? or myPID = "?" or integer( myPID ) < 2 then
    do:

      message now "I don't know my own PID:" myPID.
      message now "Refusing to create a bogus flag file.".
      if opsys = "unix" then message now "There may be a problem with $PPID -- check bin/protopenv.".
      quit.

    end.

  message now "Creating new flag file:" flgFileName myPID skip.

  output to value( flgFileName ) unbuffered append.
  put unformatted today space(1) string( time, "hh:mm:ss" ) space(1) myPID skip.
  output close.

  return.

end.


procedure searchCfg:

  define input  parameter cfgRoot   as character no-undo.
  define input  parameter cfgOption as character no-undo.
  define output parameter cfgFile   as character no-undo.

  cfgFile = ?.

  if cfgOption <> "" then
    do:
      cfgFile = substitute( "&1/&2.&3.&4", "etc", cfgRoot, cfgOption, "cfg" ).
      if dbgMode >= 5 then message now "looking for:" cfgFile.
      file-info:file-name = cfgFile.
      if file-info:full-pathname <> ? then
        do:
          if dbgMode >= 4 then message now "xfound:" cfgFile.
        end.
      cfgFile = file-info:full-pathname.
    end.

  return.

end.


procedure findCfgName:

  define input parameter cfgRoot as character no-undo.
  define input-output parameter cfgFile as character no-undo.

  define variable custId as character no-undo.

  run myCustId( output custId  ).

  /* from most specific to least specific look for:
   *
   * 1) etc/cfgRoot.pt_shortName.cfg
   * 2) etc/cfgRoot.pt_server.pt_resrcType.cfg
   * 3) etc/cfgRoot.custId.pt_resrcType.cfg
   * 4) etc/cfgRoot.pt_resrcType.cfg
   * 5) etc/cfgRoot.pt_server.cfg
   * 6) etc/cfgRoot.custId.cfg
   * 7) etc/cfgRoot.cfg
   *
   */

  run searchCfg( cfgRoot, pt_shortName, output cfgFile ).
  if cfgFile <> ? then return.

  if pt_resrcType <> "" then
    do:

      run searchCfg( cfgRoot, pt_server + "." + pt_resrcType, output cfgFile ).
      if cfgFile <> ? then return.

      run searchCfg( cfgRoot, custId + "." + pt_resrcType, output cfgFile ).
      if cfgFile <> ? then return.

      run searchCfg( cfgRoot, pt_resrcType, output cfgFile ).
      if cfgFile <> ? then return.

    end.

  run searchCfg( cfgRoot, pt_server, output cfgFile ).
  if cfgFile <> ? then return.

  run searchCfg( cfgRoot, custId, output cfgFile ).
  if cfgFile <> ? then return.

  /* naked -- no cfgOption
   */

  cfgFile = substitute( "&1/&2.&4",    "etc", cfgRoot, "", "cfg" ).
  if dbgMode >= 5 then message now "looking for:" cfgFile.
  file-info:file-name = cfgFile.
  if file-info:full-pathname <> ? then
    do:
      if dbgMode >= 4 then message now "found:" cfgFile.
      return.
    end.

end.


function leftPad returns character ( input s as character, input i as integer, input c as character ):

  return fill( c, i - length( s )) + s.

end.


function slow_xor returns integer ( x as integer, y as integer ):

  define variable i as integer no-undo.
  define variable b as integer no-undo.

  do i = 1 to 8:
    if ( get-bits( x, i, 1 ) + get-bits( y, i, 1 ) = 1 ) then
      put-bits( b, i, 1 ) = 1.
  end.

  return b.

end.


/* tom's cheesy xor
 */

/* xor_table is defined as a global shared memptr variable in protop.i */

define variable xor_init as logical no-undo.

procedure build_xor:

  define variable i as integer no-undo.
  define variable j as integer no-undo.
  define variable k as integer no-undo.

  set-size( xor_table ) = 65536.
  set-byte-order( xor_table ) = 3.

  do i = 0 to 255:
    do j = 0 to 255:
      put-byte( xor_table, ( 1 + (( i * 256 ) + j ))) = slow_xor( i, j ).
    end.
  end.

  xor_init = yes.

  return.

end.


/* gus' cheesy rc4 encryption (rc4 as implemented by gus)
 *
 * Copyright (c) 2019 Gus Bjorklund
 * Distributed under license only
 * contact gus642@gmail.com for info
 *
 * (used in ProTop with permission)
 *
 */

/* gus' xor map uses less memory but it requires more operations
 *
define variable xormap as integer no-undo extent 256 initial
  [
     0,  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15,
     1,  0,  3,  2,  5,  4,  7,  6,  9,  8, 11, 10, 13, 12, 15, 14,
     2,  3,  0,  1,  6,  7,  4,  5, 10, 11,  8,  9, 14, 15, 12, 13,
     3,  2,  1,  0,  7,  6,  5,  4, 11, 10,  9,  8, 15, 14, 13, 12,
     4,  5,  6,  7,  0,  1,  2,  3, 12, 13, 14, 15,  8,  9, 10, 11,
     5,  4,  7,  6,  1,  0,  3,  2, 13, 12, 15, 14,  9,  8, 11, 10,
     6,  7,  4,  5,  2,  3,  0,  1, 14, 15, 12, 13, 10, 11,  8,  9,
     7,  6,  5,  4,  3,  2,  1,  0, 15, 14, 13, 12, 11, 10,  9,  8,
     8,  9, 10, 11, 12, 13, 14, 15,  0,  1,  2,  3,  4,  5,  6,  7,
     9,  8, 11, 10, 13, 12, 15, 14,  1,  0,  3,  2,  5,  4,  7,  6,
    10, 11,  8,  9, 14, 15, 12, 13,  2,  3,  0,  1,  6,  7,  4,  5,
    11, 10,  9,  8, 15, 14, 13, 12,  3,  2,  1,  0,  7,  6,  5,  4,
    12, 13, 14, 15,  8,  9, 10, 11,  4,  5,  6,  7,  0,  1,  2,  3,
    13, 12, 15, 14,  9,  8, 11, 10,  5,  4,  7,  6,  1,  0,  3,  2,
    14, 15, 12, 13, 10, 11,  8,  9,  6,  7,  4,  5,  2,  3,  0,  1,
    15, 14, 13, 12, 11, 10,  9,  8,  7,  6,  5,  4,  3,  2,  1,  0
  ]
.
 *
 */

procedure gcr_encrypt:

    define input parameter keyText  as character no-undo.
    define input parameter data     as memptr    no-undo.

    define variable numBytes   as integer no-undo.
    define variable byteNumber as integer no-undo.

    define variable s  as integer no-undo extent 256.
    define variable i  as integer no-undo.
    define variable j  as integer no-undo.
    define variable t  as integer no-undo.
    define variable k  as integer no-undo.
    define variable kl as integer no-undo.
    define variable z  as integer no-undo.

    if xor_init = no then run build_xor.

    do i = 0 to 255:
        s[i + 1] = i.
    end.

    j = 0.
    kl = length( keyText, "character" ).
    do i = 1 to 256:

      assign
        k = ( i - 1 ) modulo kl
        j = j + s[i] + asc( substring( keyText, k + 1, 1 ))
        j = j modulo 256
        z = j + 1
        t = s[i]
        s[i] = s[z]
        s[z] = t
      .

    end.

    assign
      numBytes = get-size( data ) 	/* - 1 */				/* COPY-LOB doesn't terminate memptrs with NUL				*/
      j = 0
    .

    do byteNumber = 1 to numBytes:

      assign
        i = ( byteNumber modulo 256 ) + 1
        j = (( s[i] + j ) modulo 256 )
        z = j + 1
        t = s[i]
        s[i] = s[z]
        s[z] = t
        k = s[ ((s[i] + s[z]) modulo 256 ) + 1 ]
        t = get-byte( data, byteNumber )
      .

      put-byte( data, byteNumber ) = get-byte( xor_table, ( 1 + (( k * 256 ) + t ))).	/* tom's xor method (about 20% faster - but uses 64k of RAM vs ~1k for the array of 256 integers) */

      /* gus' xor method
       *
       * put-byte( data, bytenumber) = xormap[((( t modulo 16 ) * 16 ) + ( k modulo 16 )) + 1] + ( xormap[(( get-bits( t, 5, 4 ) * 16 ) + get-bits( k, 5, 4 )) + 1] * 16 ).
       *
       */

    end.

end.


function m2hex returns longchar ( input m as memptr ):

  define variable i   as integer no-undo.
  define variable j   as integer no-undo.
  define variable n   as integer no-undo.
  define variable n1  as integer no-undo.
  define variable n2  as integer no-undo.

  define variable nb  as integer   no-undo.
  define variable h   as character no-undo initial "0123456789abcdef".
  define variable ret as longchar  no-undo initial "".

  nb = get-size( m ).			/* - 1. */				/* COPY-LOB doesn't terminate memptrs with NUL				*/
  do n = 1 to nb:
    j = j + 1.
    i = get-byte( m, n ).
    n2 = get-bits( i, 5, 4 ).
    n1 = get-bits( i, 1, 4 ).
    ret = ret + substring( h, n2 + 1, 1 ) + substring( h, n1 + 1, 1 ).
  end.

  /* message "m2hex():" "nb:" nb "n:" n "j:" j "i:" i chr( i ). */

  if ret = ? then ret = "".

  return ret.

end.


function hex2m returns memptr ( input x as longchar ):

  define variable i   as integer no-undo.
  define variable n   as integer no-undo.
  define variable n1  as integer no-undo.
  define variable n2  as integer no-undo.

  define variable m   as memptr no-undo.

  define variable p   as integer no-undo.
  define variable b   as integer no-undo.

  n = length( x ).

  set-size( m ) = integer( n / 2 ).	 /* + 1. */				/* copy-lob does NOT add a NUL terminator to memptrs			*/
  set-byte-order( m ) = 3.

  do i = 1 to n by 2:

    assign
      p  = p + 1
      n1 = ( asc( string( substring( x, i, 1 ))) - 48 )
      n2 = ( asc( string( substring( x, i + 1, 1 ))) - 48 )
    .

    if n1 > 10 then n1 = n1 - 39.
    if n2 > 10 then n2 = n2 - 39.

    b = (( n1 * 16 ) + n2 ).

    /* message p b. */

    put-byte( m, p ) = b.

  end.

  return m.

end.


function text2GCRHex returns longchar ( input plainText as longchar, input keyText as character ):

  define variable hexText as longchar no-undo.
  define variable m       as memptr   no-undo.
  define variable sz      as integer  no-undo.

  /*
   * sz = length( plainText ) + 1.						/* copy-lob does NOT add a NUL terminator to memptrs			*/
   * set-size( m ) = sz.							/* copy-lob also does its own memory allocation, so this is not needed	*/
   */

  set-byte-order( m ) = 3.

  /* message length( plainText ) sz "m, pre:" get-size( m ). */

  /* put-string( m, 1, length( plainText ) + 1 ) = plainText. */		/* replaced put-strng() with copy-lob in order to handle larger data	*/

  copy-lob from plainText to m no-error.

  /* message length( plainText ) sz "m, post:" get-size( m ). */

  run gcr_encrypt( keyText, m ).

  hexText = m2Hex( m ).

  set-size( m ) = 0.

  return hexText.

end.


function hexGCR2Text returns longchar ( input hexText as longchar, input keyText as character ):

  define variable outputText as longchar  no-undo.
  define variable z          as memptr    no-undo.
  define variable n          as integer   no-undo.

  if hexText > "" then
    do:

      n = length( hexText ).

      if substring( hexText, n, 1 ) = "~n" then					/* oops! someone edited the file and saved it with an editor that fixes	*/
        hexText = substring( hexText, 1, n - 1 ).				/* crappy files. they will be disappointed if they changed any of the	*/
										/* hex values ;) but curiosity is fine so we will quietly handle this	*/
      z = hex2m( hexText ).

      run gcr_encrypt( keyText, z ).

      /* outputText = get-string( z, 1, integer( length( hexText ) / 2 )). */	/* replaced get-string() with copy-lob in order to handle larger data	*/

      copy-lob from z to outputText no-error.

      set-size( z ) = 0.

    end.

  if outputText = ? then outputText = "".

  return outputText.

end.



/* return the next power of 2 (NOT the nearest)
 */

function nextPower2 returns decimal ( input x as decimal ):

  define variable p as integer no-undo.

  if x < 0 then return ?.							/* this function is only defined for positive values			*/
  
  do while exp( 2, p ) < x:
    p = p + 1.
  end.

  return exp( 2, p ).
  
end.


/* calculate the "ideal" rows per block
 *
 * using the average record size assumes a gaussian ("bell curve") distribution
 * of record sizes.  this is usually a reasonable assumption but there can be
 * exceptions.
 *
 * bsz = block size in bytes
 * avgRec = average record size
 *
 */

function idealRPB returns integer ( input bsz as integer, input avgRec as decimal ):

  return integer( min( 256, nextPower2(( bsz / ( avgRec + 20 ))))).		/* a fairly simple rule, but reasonably effective			*/

end.


/* The date-time field in certain VSTs looks like: Mon Jan 14 03:33:08 2013
 *
 * usage:
 *
 *   datetime( fixDateString( _Checkpoint._Checkpoint-Time ))
 * 
 * known fields:
 * 
 *   _Checkpoint._Checkpoint-Time
 *   _Checkpoint._Checkpoint-Len
 *   _Connect._Connect-Time
 *   _Connect._Connect-CacheLastUpdate
 *   _dbStatus._dbStatus-fbDate
 *   _dbStatus._dbStatus-ibDate
 *
 * not sure if month names need translation :(
 *
 */

function fixDateString returns character ( input dx as character ):

  define variable d as character no-undo.
  define variable m as character no-undo.
  define variable y as character no-undo.
  define variable t as character no-undo.

  assign
    m = string( lookup( substring( dx, 5, 3 ), "Jan,Feb,Mar,Apr,May,Jun,Jul,Aug,Sep,Oct,Nov,Dec" ), "99" )
    d = string( integer( trim( substring( dx, 9, 2 ))), "99" )
    y = substring( dx, 21 )
    t = substring( dx, 12, 8 )
  .

  if       session:date-format = "ymd" then return substitute( "&1-&2-&3 &4", y, m, d, t ).
   else if session:date-format = "mdy" then return substitute( "&1-&2-&3 &4", m, d, y, t ).
   else if session:date-format = "dmy" then return substitute( "&1-&2-&3 &4", d, m, y, t ).
   else
    do:
      message "Unsupported session:date-format" session:date-format view-as alert-box.
      quit.
    end.

end.


/* logMsg
 *
 * three ways to call it:
 *
 * logMsg( msgLevel, messageText ).
 * publish "logMsg" ( msgLevel, messageText ).
 * run doLogMsg ( msgLevel, messageText ).
 *
 */

define stream logStream.

define variable logMsgFileName as character no-undo.

procedure setLogName:

  define input parameter logName as character no-undo.

  logMsgFileName = logName.

  return.
end.


procedure doLogMsg:

  define input parameter msgLevel as integer   no-undo.
  define input parameter msgText  as character no-undo.

  if logMsgFileName = "" or logMsgFileName = ? then
    run setLogName( substitute( "&1/noname.log", pt_tmpdir )). 

  if dbgMode >= msgLevel then
    do:
      output stream logStream to value( logMsgFileName ) append.
  /*  put    stream logStream unformatted today " " string( time, "hh:mm:ss" ) " " msgText skip.	*/
      put    stream logStream unformatted now " " msgText skip.
      output stream logStream close.
    end.

  return.

end.

function logMsg returns logical ( input msgLevel as integer, input msgText as character ):

  run doLogMsg( msgLevel, msgText ).

  return true.

end.


procedure getAgentByPID:

  define input  parameter PID         as character no-undo.			/* target process Id						*/
  define output parameter commandLine as character no-undo.			/* the command line of the target process			*/

  define variable execCmdLine as character no-undo.

  if dbgMode >= 3 then
    message now "getAgentByPID, looking for processId:" PID.

  if opsys = "unix" then
    do:

      execCmdLine = substitute( pt_getPID, PID ).

      if dbgMode >= 4 then
        message now "getAgentByPID, exec: " execCmdLine.

      input stream inStrm through value( execCmdLine ).				/* depending on the command there _might_ be a header line	*/
      repeat:									/* so REPEAT and read them all, if the last line has the PID	*/
        import stream inStrm unformatted commandLine.				/* we have found what we came for				*/
        if dbgMode >= 5 then
          message now "getAgentByPID, input line: " commandLine.
      end.
      input stream inStrm close.

      /* this assumes that all flavors of pt_getPID return the PID in column 2
       */

      if PID = entry( 1, trim( substring( commandLine, index( commandLine, " " ))), " " ) then
        do:
          message now substitute( "getAgentByPID, PID &1 was found: &2", PID, commandLine ).
        end.

    end.
   else
    do:

      run getProcessList( "_progres,p util~\pt3agent" ).

      find tt_process where tt_process.pid = PID no-error.

      if available( tt_process ) then
        commandLine = tt_process.cmdLine.
       else
        do:
          message now substitute( "getAgentByPID, PID &1 was not found: &2", PID ).
          commandLine = "".							/* do not return a junk string!					*/
        end.
    end.

  if dbgMode > 3 and commandLine <> "" then
    message now substitute( "getAgentByPID, PID &1 is running: &2", PID, commandLine ).

  return.

end.


procedure getAgentByName:

  define input  parameter frndName    as character no-undo.			/* target agent friendlyName					*/
  define output parameter osPID       as character no-undo.			/* the PID of the target process				*/
  define output parameter commandLine as character no-undo.			/* the command line of the target process			*/

  define variable i as integer no-undo.

  if opsys = "unix" then
    run getProcessList( substitute( " -param &1 ", frndName )).
   else
    run getProcessList( "_progres, -p util~\pt3agent," + substitute( " -param &1 ", frndName )).

  osPID = "".
  commandLine = "".

  i = 0.
  for each tt_process:
    if dbgMode >= 5 then message now "getAgentByName pid =" tt_process.pid "commandLine =" tt_process.cmdLine.
    osPID = tt_process.pid.
    commandLine = tt_process.cmdLine.
    i = i + 1.
  end.

  if i = 0 then
    message now "getAgentByName, no processes found".
   else if i > 1 then
    message now "getAgentByName," i "processes found! (should be 1)".
   else if i = 1 and dbgMode >= 5 then
    message now "getAgentByName, found:" osPID commandLine.

  return.

end.


procedure getProcessList:

  define input parameter matchList as character no-undo.			/* "_progres,p util~\pt3agent"				*/

  define variable commandLine as character no-undo.
  define variable execCmdLine as character no-undo.
  define variable posPID      as integer   no-undo.				/* position of the ProcessID field			*/
  define variable xpid        as character no-undo.

  define variable i  as integer no-undo.
  define variable n  as integer no-undo.
  define variable ok as logical no-undo.

  empty temp-table tt_process.

  n = num-entries( matchList ).

  if opsys = "unix" then
    do:

      execCmdLine = pt_getAgentList.						/* 'ps -C _progres -f | grep "p util/pt3agent"'			*/

      if dbgMode >= 3 then
        message now "getProcessList, exec: " execCmdLine.

      input stream inStrm through value( execCmdLine ).				/* depending on the command there _might_ be a header line	*/
      inLoop: repeat:								/* so REPEAT and read them all					*/
										/* the pid field is character so a header value hurts nothing	*/
        import stream inStrm unformatted commandLine.
        if dbgMode >= 5 then
          message now "getProcessList: " commandLine.

        do i = 1 to n:
          if index( commandLine, entry( i, matchList )) = 0 then
            do:
              if dbgMode >= 5 then
                message now substitute( 'getProcessList: rejected "&1" was not matched', entry( i, matchList )). 
              next inLoop.
            end.
        end.

        /* this assumes that all flavors of pt_getPID return the PID in column 2
         */

        xpid = entry( 1, trim( substring( commandLine, index( commandLine, " " ))), " " ).

        find tt_process where tt_process.pid = xpid no-error.
        if available tt_process then
          do:
            message now "getProcessList: PID already exists in tt_process!" commandLine.
            next.
          end.

        create tt_process.
        assign
          tt_process.pid = xpid
          tt_process.cmdLine = commandLine
        .

      end.
      input stream inStrm close.

    end.
   else
    do:

      if pt_usewmic4ps = yes then
        do:

          /* OE10 and earlier we still need to use wmic :(
           */

          execCmdLine =  "wmic process get commandline,name,processid".

          /* yes, the output lines are obnoxiously wide
           *
           * CommandLine                                                                                                                                                           Name                                 ProcessId  
           *                                                                                                                                                                       System Idle Process                  0          
           *                                                                                                                                                                       System                               4          
           *                                                                                                                                                                       Registry                             92         
           * . . .
           * C:\Windows\system32\cmd.exe   /K C:\Progress\OpenEdge\OE122\DLC\bin\proenv.bat psc                                                                                    cmd.exe                              8216       
           * "C:\Windows\System32\SecurityHealthSystray.exe"                                                                                                                       SecurityHealthSystray.exe            9484       
           * "C:\Windows\System32\vm3dservice.exe" -u                                                                                                                              vm3dservice.exe                      1576       
           * "C:\Program Files\VMware\VMware Tools\vmtoolsd.exe" -n vmusr                                                                                                          vmtoolsd.exe                         4436       
           * "C:\Users\vpnuser\AppData\Local\Microsoft\OneDrive\OneDrive.exe" /background                                                                                          OneDrive.exe                         8492       
           * "C:\Program Files (x86)\Cisco\Cisco AnyConnect Secure Mobility Client\vpnui.exe" -minimized                                                                           vpnui.exe                            7648       
           * "C:\Program Files\Windows NT\Accessories\wordpad.exe" /restart e76e55ca-3a14-4cf4-83fd-157bad966b7a                                                                   wordpad.exe                          9000       
           * "C:\Program Files (x86)\CheckPoint\Endpoint Connect\TrGUI.exe"                                                                                                        TrGUI.exe                            9152       
           * C:\Windows\system32\svchost.exe -k PrintWorkflow -s PrintWorkflowUserSvc                                                                                              svchost.exe                          4940       
           * wmic  process get processid,name,commandLine                                                                                                                          WMIC.exe                             6268       
           * 
           */

          if dbgMode >= 3 then
            message now "getProcessList, wmic: " execCmdLine " matchList =" matchList.

          input stream inStrm through value( execCmdLine ).

          /* the first line should be a header line
           */

          inLoop: repeat:

            import stream inStrm unformatted commandLine.
            if dbgMode >= 5 or posPID < 0 then
              message now "getProcessList, wmic:" commandLine.

            if posPID = 0 then
              do:
                posPID = index( commandLine, pt_wmicPIDlbl ).
                if posPID = 0 then
                  do:
                    message now "getProcessList, wmic, not a header line:" commandLine.
                    posPID = -1.
                    next.
                  end.
              end.

            if posPID > 0 then
              xpid = trim( substring( commandLine, posPID )).
             else
              next.

            do i = 1 to n:
              if index( commandLine, entry( i, matchList )) = 0 then
                do:
                  if dbgMode >= 5 then
                    message now substitute( 'getProcessList: rejected "&1" was not matched', entry( i, matchList )). 
                  next inLoop.
                end.
            end.

            find tt_process where tt_process.pid = xpid no-error.
            if available tt_process then
              do:
                message now "getProcessList: PID already exists in tt_process!" commandLine.
                next.
              end.

            create tt_process.
            assign
              tt_process.pid = xpid
              tt_process.cmdLine = commandLine
            .

          end.

          input stream inStrm close.

        end.
       else
        do:

          if dbgMode >= 3 then
            message now "getProcessList, dll call, matchList =" matchList "dbgMode =" dbgMode.

          run lib/getprocdll.p ( matchList ).

        end.

    end.

  return.

end.


/* determine if we are running with user input or redirected input
 *
 */

function isatty returns logical ():

  define variable result     as logical   no-undo.
  define variable tty        as character no-undo.
  define variable fileHandle as int64     no-undo.
  define variable fileType   as int64     no-undo.

  result = false.

  if opsys = "unix" then
    do:

      input stream inStrm through value( "tty" ).
      import stream inStrm unformatted tty.
      input stream inStrm close.
      if tty begins "/dev/" then
        result = true.

    end.
   else
    do:

      /* Windows stdin = -10	*/

      run getStdHandle( -10, output fileHandle ).

      run getFileType( fileHandle, output fileType ).

      /* 0x0000 = unknown
       * 0x0001 = disk
       * 0x0002 = character (CON or LPT etc)
       * 0x0003 = pipe
       * 0x8000 = remote (unused?)
       */

      if fileType = 2 then
        result = true.

    end.

  return result.

end.


/* reformat TT fields for alternate display widths
 */

function altFormat returns logical ( a as character, b as handle, n as character, f as character ):

  assign
    b:buffer-field( n ):format       = f
    b:buffer-field( n ):private-data = a
  .

  return true.

end.


procedure getConfig:

  define input  parameter cfgName as character no-undo.
  define output parameter cfgBody as character no-undo.

  define variable xLine   as character no-undo.
  define variable dName   as character no-undo initial ?.
  define variable respHdr as character no-undo.

  /* grab width specific configs if they exist
   */

  if pt_altWidth <> "" then
    do:
      dName = search( replace( cfgName, ".json", pt_altWidth + ".json" )).
      if dName = ? then dName = search( "ptdefs/" +  replace( cfgName, ".json", pt_altWidth + ".json" )).
    end.

  if dname = ? then
    do:
      dName = search( cfgName ).						/* first check the PROPATH			*/
      if dName = ? then dName = search( "ptdefs/" + cfgName ).			/* maybe it is in a sub-dir?			*/
    end.

  if dName = ? then
    do:
      message now "Could not find: ptdefs/" + cfgName view-as alert-box.	/* this might deserve a QUIT?			*/
    end.
   else
    do:

      if dbgMode >= 3 then message now "reading" dName.

      /* avoiding COPY-LOB allows support for v9 but limits the size of the JSON file to ~ 30,000 bytes
       */

      cfgBody = "".
      input stream inStrm from value( dName ).
      repeat:
        import stream inStrm unformatted xLine.
        cfgBody = cfgBody + xLine.
      end.
      input stream inStrm close.

    end.

  return.

end.


/* end protoplib.p */
