/* ptcolor.p
 *
 * show protop color assignments
 *
 */

{lib/protop.i}
{lib/dynscreen.i}

define variable cName   as character format "x(12)" no-undo extent 10.
define variable cNum    as integer   format ">>9"   no-undo extent 10.

define variable clrFunction as character no-undo initial "header,label,browseLabel,browseTitle,sortUp,sortDown,data,good,warning,alert".

define variable clrNumList     as character no-undo initial "2,8,8,11,19,30,2,6,12,5".
define variable monochromeList as character no-undo initial "0,0,0,0,0,0,0,0,0,0".
define variable accessibleList as character no-undo initial "2,8,8,11,18,1,2,22,1,3".
define variable darkModeList   as character no-undo initial "2,8,8,11,19,30,2,6,12,5".

define variable clrEnv      as character format "x(50)" no-undo.

if session:window-system <> "tty" then
  do:
    clrNumList = "0,0,0,0,0,0,3,2,12,13".			/* etc/protop.ini [windows gui]		*/
  end.
 else
  do:
    if opsys <> "unix" then
      assign							/* etc/protop.ini [winchar]		*/
        clrNumList = "7,8,8,11,10,3,15,6,4,5"
        accessibleList = "7,8,8,11,16,24,15,48,28,21"
      .
  end.

if os-getenv( "PTCOLORS" ) <> ? and num-entries( os-getenv( "PTCOLORS" )) = num-entries( clrFunction ) then
  clrNumList = os-getenv( "PTCOLORS" ).

session:add-super-procedure( this-procedure ).

subscribe to "protop_colors" anywhere run-procedure "ptcolors".

return.


function getColorNum returns integer ( colorCode as character ):

  define variable colorNum as integer no-undo.
  
  if colorCode = "" or colorCode = ? then colorCode = "normal".

  colorNum = integer( entry( lookup( colorCode, clrFunction ), clrNumList )) no-error.

  publish "logMsg" ( 8, substitute( "Looking for: &1 clrFunction = &2 clrNumList = &3 result: &4", colorCode, clrFunction, clrNumList, lookup( colorCode, clrFunction ))).

  if error-status:error then 
   return 0.
  else
   return colorNum. 

end.


procedure clrList2Array:

  define input  parameter clrList  as character no-undo.
  define output parameter numArray as integer extent 10 no-undo.

  define variable ii as integer no-undo.

  do ii = 1 to 10:
    numArray[ii] = integer( entry( ii, clrList )) no-error.
  end.

  return.

end.


define frame widgetTree with down overlay row 1 no-labels.

procedure showTree:

  define input parameter w as handle  no-undo.
  define input parameter d as integer no-undo.

  if valid-handle( w ) then
    do:
      display
        int64( w )
        d
        substitute( "&1&2", fill( " ", d * 2 ), w:name ) format "x(30)"
        w:type
        w:index
        w:value
        w:dcolor
       with
        frame widgetTree
       no-error
      .
      down with frame widgetTree.
    end.

  return.

end.


procedure walkTree:				/* the order is kind of backwards... but that doesn't really matter	*/

  define input parameter w as handle  no-undo.
  define input parameter d as integer no-undo.

  define variable n as handle no-undo.

  n = w:first-child no-error.
  if valid-handle( n ) then
    run walkTree( n, d + 1 ).

  n = w:next-sibling no-error.
  if valid-handle( n ) then
    run walkTree( n, d ).

  run showTree( w, d ).

  return.

end.


/* walk the widget tree so that we can update cNum and set the dcolors
 *
 * this is necessary because EDITING blocks are deprecated so support
 * for things like changing dcolor while inside one was never added
 *
 * the issue with cNum and clrEnv seems less like a deprecation issue
 * but INPUT( cNum ) doesn't work so here we are
 *
 */

procedure tweakTree:

  define input parameter w    as handle no-undo.
  define input parameter wIdx as integer no-undo.
  define input parameter dClr as integer no-undo.

  define variable n as handle no-undo.

  n = w:first-child no-error.
  if valid-handle( n ) then
    run tweakTree( n, wIdx, dClr ).

  n = w:next-sibling no-error.
  if valid-handle( n ) then
    run tweakTree( n, wIdx, dClr ).

  if w:type = "fill-in" and w:name = "cNum" and w:index = wIdx then
    cNum[wIdx] = dClr.									/* this is ridiculous - but I can't find a better way :(	*/

  if w:type = "fill-in" and ( w:name = "cNum" or w:name = "cName" ) and w:index > 0 then
    do:
      if session:window-system = "tty" then
        w:dcolor  = cNum[w:index].
       else
        w:fgcolor = cNum[w:index].
    end.

  return.

end.


function getClrEnv returns character ():			/* passing INPUT( cNum ) doesn't work, you get the initial value not the INPUT value	*/

  define variable i as integer no-undo.

  clrEnv = "".

  do i = 1 to num-entries( clrFunction ):
    clrEnv = substitute( "&1,&2", clrEnv, cNum[i] ).
  end.

  if opsys = "unix" then
    return 'export PTCOLORS="' + trim( clrEnv, "," ) + '"'.
   else
    return 'set PTCOLORS="' + trim( clrEnv, "," ) + '"'.

end.


procedure ptcolors:

  define variable f as handle  no-undo.
  define variable i as integer no-undo.

  define variable ptString   as character no-undo format "x(36)".
  define variable gridHeader as character no-undo format "x(56)".

  if opsys = "unix" then
    assign
      ptString   = "$PROTOP/etc/protermcap.protop"
      gridHeader = "Basic  Plain   Bold  Italic UndrLn   Rev   Misc1  Misc2"
    .
   else
    assign
      ptString   = "%PROTOP%~\etc~\protop.ini"
      gridHeader = ""
    .

  do i = 1 to num-entries( clrFunction ):
    assign
      cName[i] = entry( i, clrFunction )
      cNum[i]  = getColorNum( cName[i] )
    .
  end.

  clrEnv = getClrEnv().

  do on error undo, leave
     on endkey undo, leave:

    display
      skip(1)
      "System Colors:" at 2 " Normal (0) " dcolor 0 fgcolor 0 at 16 " Input (1) " dcolor 1 fgcolor 1 at 30 " Messages (2) " dcolor 2 fgcolor 2 at 42
      skip(1)
      gridHeader at 20
      "  "  " " dcolor cNum[1] cName[1]  dcolor cNum[1]  at 3  cNum[1]  dcolor  cNum[1] at 16    " 0 "  dcolor  0 fgcolor  0 at 22  "  8 " dcolor  8 fgcolor  8 at 28  " 16 " dcolor 16 fgcolor 16 at 35  " 24 " dcolor 24 fgcolor 24 at 42  " 32 "  dcolor 32 fgcolor 32 at 49  " 40 "  dcolor 40 fgcolor 40 at 57  " 48 " dcolor 48 fgcolor 48 at 64  " 56 " dcolor 56 fgcolor 56 at 71
      "  "  " " dcolor cNum[2] cName[2]  dcolor cNum[2]  at 3  cNum[2]  dcolor  cNum[2] at 16    " 1 "  dcolor  1 fgcolor  1 at 22  "  9 " dcolor  9 fgcolor  9 at 28  " 17 " dcolor 17 fgcolor 17 at 35  " 25 " dcolor 25 fgcolor 25 at 42  " 33 "  dcolor 33 fgcolor 33 at 49  " 41 "  dcolor 41 fgcolor 41 at 57  " 49 " dcolor 49 fgcolor 49 at 64  " 57 " dcolor 57 fgcolor 57 at 71
      "  "  " " dcolor cNum[3] cName[3]  dcolor cNum[3]  at 3  cNum[3]  dcolor  cNum[3] at 16    " 2 "  dcolor  2 fgcolor  2 at 22  " 10 " dcolor 10 fgcolor 10 at 28  " 18 " dcolor 18 fgcolor 18 at 35  " 26 " dcolor 26 fgcolor 26 at 42  " 34 "  dcolor 34 fgcolor 34 at 49  " 42 "  dcolor 42 fgcolor 42 at 57  " 50 " dcolor 50 fgcolor 50 at 64  " 58 " dcolor 58 fgcolor 58 at 71
      "  "  " " dcolor cNum[4] cName[4]  dcolor cNum[4]  at 3  cNum[4]  dcolor  cNum[4] at 16    " 3 "  dcolor  3 fgcolor  3 at 22  " 11 " dcolor 11 fgcolor 11 at 28  " 19 " dcolor 19 fgcolor 19 at 35  " 27 " dcolor 27 fgcolor 27 at 42  " 35 "  dcolor 35 fgcolor 35 at 49  " 43 "  dcolor 43 fgcolor 43 at 57  " 51 " dcolor 51 fgcolor 51 at 64  " 59 " dcolor 59 fgcolor 59 at 71
      "  "  " " dcolor cNum[5] cName[5]  dcolor cNum[5]  at 3  cNum[5]  dcolor  cNum[5] at 16    " 4 "  dcolor  4 fgcolor  4 at 22  " 12 " dcolor 12 fgcolor 12 at 28  " 20 " dcolor 20 fgcolor 20 at 35  " 28 " dcolor 28 fgcolor 28 at 42  " 36 "  dcolor 36 fgcolor 36 at 49  " 44 "  dcolor 44 fgcolor 44 at 57  " 52 " dcolor 52 fgcolor 52 at 64  " 60 " dcolor 60 fgcolor 60 at 71
      "  "  " " dcolor cNum[6] cName[6]  dcolor cNum[6]  at 3  cNum[6]  dcolor  cNum[6] at 16    " 5 "  dcolor  5 fgcolor  5 at 22  " 13 " dcolor 13 fgcolor 13 at 28  " 21 " dcolor 21 fgcolor 21 at 35  " 29 " dcolor 29 fgcolor 29 at 42  " 37 "  dcolor 37 fgcolor 37 at 49  " 45 "  dcolor 45 fgcolor 45 at 57  " 53 " dcolor 53 fgcolor 53 at 64  " 61 " dcolor 61 fgcolor 61 at 71
      "  "  " " dcolor cNum[7] cName[7]  dcolor cNum[7]  at 3  cNum[7]  dcolor  cNum[7] at 16    " 6 "  dcolor  6 fgcolor  6 at 22  " 14 " dcolor 14 fgcolor 14 at 28  " 22 " dcolor 22 fgcolor 22 at 35  " 30 " dcolor 30 fgcolor 30 at 42  " 38 "  dcolor 38 fgcolor 38 at 49  " 46 "  dcolor 46 fgcolor 46 at 57  " 54 " dcolor 54 fgcolor 54 at 64  " 62 " dcolor 62 fgcolor 62 at 71
      "  "  " " dcolor cNum[8] cName[8]  dcolor cNum[8]  at 3  cNum[8]  dcolor  cNum[8] at 16    " 7 "  dcolor  7 fgcolor  7 at 22  " 15 " dcolor 15 fgcolor 15 at 28  " 23 " dcolor 23 fgcolor 23 at 35  " 31 " dcolor 31 fgcolor 31 at 42  " 39 "  dcolor 39 fgcolor 39 at 49  " 47 "  dcolor 47 fgcolor 47 at 57  " 55 " dcolor 55 fgcolor 55 at 64  " 63 " dcolor 63 fgcolor 63 at 71
      "  "  " " dcolor cNum[9] cName[9]  dcolor cNum[9]  at 3  cNum[9]  dcolor  cNum[9] at 16
      "  "  " " dcolor cNum[10]cName[10] dcolor cNum[10] at 3  cNum[10] dcolor cNum[10] at 16
      skip(1)
/*
 *    "  If you would like to change the startup colors copy this to bin/localenv:" skip
 *         clrEnv at 4 skip
 *    skip(1)
 *    "  The overall background color does not change. That is set at startup and cannot be changed at runtime." skip
 *    skip(1)
 *    "       b = boring colors  " skip 
 *    "       c = colorful       " skip
 *    "       m = monochrome     " skip
 *    "       d = dark mode      (may be useful if your background is dark)  " skip
 *    skip(1)
 */
      "   c = colorful    m = monochrome    a = accessible    d = dark mode  " skip
      "   x = save & exit  q = quit (no save)                                " skip
     with
      frame getColors
      title dcolor 68 " ProTop Colors "
      row 4
      column ( if current-window:width-chars < 160 then 1 else 40 )
      width /* 128 */ 80
      no-labels
      overlay
/*
 *    dcolor 67			/* chui default background and text	- mustard on blue	*/
 *    pfcolor 71 		/* chui defauly INPUT			- white on blue 	*/
 */
    .

    f = frame getColors:handle.

/*
 *  run showTree( f, 0 ).
 *  run walkTree( f:first-child, 1 ).
 *  pause.
 */

    update
       cNum
     with
      frame getColors
     editing:

       readkey.

       if lookup( keyfunction( lastkey ), "c,m,a,d" ) > 0 then
         do:

           run tweakTree( f:first-child, frame-index, integer( frame-value )).

           if       keyfunction( lastkey ) = "c" then 	/* colorful		*/
             do:
               run clrList2Array ( clrNumList, output cNum ).
               display cNum with frame getColors.
             end.
            else if keyfunction( lastkey ) = "m" then 	/* monochrome		*/
             do:
               run clrList2Array ( monochromeList, output cNum ).
               display cNum with frame getColors.
             end.
            else if keyfunction( lastkey ) = "a" then 	/* accessible colors	*/
             do:
               run clrList2Array ( accessibleList, output cNum ).
               display cNum with frame getColors.
             end.
            else if keyfunction( lastkey ) = "d" then 	/* dark mode		*/
             do:
               run clrList2Array ( darkModeList, output cNum ).
               display cNum with frame getColors.
             end.

           run tweakTree( f:first-child, frame-index, integer( frame-value )).

           clrEnv = getClrEnv().
           display clrEnv with frame getColors.

         end.
        else if keyfunction( lastkey ) = "x" then 	/* save and exit	*/
         apply "go".
        else if lookup( keyfunction( lastkey ), "f4,q,end-error" ) > 0 then	/* quit, no save	*/
         apply "end-error".
        else
         apply lastkey.

     end.
    .

    /* save the values & make it real!
     */

    clrNumList = "".
    do i = 1 to num-entries( clrFunction ):
      clrNumList = substitute( "&1,&2", clrNumList, cNum[i] ).
    end.
    clrNumList = trim( clrNumList, "," ).

    /* it is tempting to automatically source this in protopenv much like with localenv
     * HOWEVER - the idea is to produce a script to create user preferences that would
     * then be invoked by each user, probably at login time (or in similar fashion) OR
     * to be specific to an "environment" (like PROD vs DEV) so automatically loading
     * probably isn't such a good idea
     */

    if opsys = "unix" then
      do:
        output to value( "bin/colors.sh" ).
        put unformatted substitute( 'export PTCOLORS="&1"', clrNumList ) skip.
        output close.
      end.
     else
      do:
        output to value( "bin~\colors.bat" ).
        put unformatted substitute( 'set PTCOLORS=&1', clrNumList ) skip.	/* beware windows quoting :(	*/
        output close.
      end.

    output close.

  end.

  hide frame getColors.

  return.

end.
