@ECHO OFF
::
::   Name: maintenance.cmd
::   Author: White Star Software
::   Date: 25 January 2017
::
::   Purpose: Create an outage on the ProTop Dashboard
::
::
::   Parameters: Action: "On", "Off", "Status"
::               Length: N (Length of outage in hours if arg1 = "on")
::
:: Fixes 2019/03/06: Now uses PHP api and output date independantly of locale
:: Fixes 2019/11/18: use PROXY in curl if set
:: Fixes 2020/04/08: replace case with util/maintenance.p and handles PROXY

:: Delay variable expansion to runtime

Setlocal EnableDelayedExpansion

:: Grab environment vars

CALL "#ENVPT3#\bin\protopenv.bat"

cd /d %PROTOP%

:: VARIABLES

:: Switch these around for normal use
::SET EKKO=echo
SET EKKO=

SET AND=IF
SET MLOG="%LOGDIR%\maintenance.log"
SET MFLAG="%TMPDIR%\MAINTENANCE"
SET "USAGE1=Usage: maintenance.cmd 'on' n ^(Hours^)^|off^|status"
SET "USAGE2=   ie: maintenance.cmd on 3"
SET "USAGE3=       maintenance.cmd off"
SET "USAGE4=       maintenance.cmd status"

::   Process Parameters

IF "%1" == "" (
   CALL :SUB_USAGE
   GOTO :ABEND
)

SET ACTION=%1

IF NOT "%ACTION%"=="on" %AND% NOT "%ACTION%"=="off" %AND% NOT "%ACTION%"=="status" (
   CALL :SUB_USAGE
   GOTO :ABEND
)

IF "%ACTION%"=="on" (

   IF "%2"=="" (
      CALL :SUB_USAGE
      GOTO :ABEND
   )
   echo %2| findstr /r "^[1-9][0-9]*$">NUL

   IF NOT %errorlevel% equ 0 (
      echo "Error: Duration must be a positive integer"
      GOTO :ABEND
    )
   SET DURATION=%2
)

cd "%PROTOP%"

IF NOT EXIST "%TMPDIR%" md "%TMPDIR%"
IF NOT EXIST "%LOGDIR%" md "%LOGDIR%"

IF NOT EXIST "%PROTOP%\etc\custid.cfg" (
   echo "Error: Cannot find custid.cfg - exiting"
   GOTO :ABEND
)

FOR %%a in ("%PROTOP%\etc\custid.cfg") do if %%~za equ 0 (
   echo "Error: custid.cfg = 0 bytes"
   GOTO :ABEND
)

SET /P CUSTID=<"%PROTOP%\etc\custid.cfg


:: Send the parameters to progress client session to process the action

SET PARAM="%ACTION%|%DURATION%|%TMPDIR%|%LOGDIR%|%CUSTID%|%PTHOST%"

:: add %PFLIST% defined in protopenv.bat to pull in -certstorepath if used (in [custid].pf), with localenv:USESSL=y
%DLC%\bin\_progres.exe -b -p util\maintenance.p -basekey "INI" -ininame etc\protop.ini %PFLIST% -d ymd -T %TMPDIR% -param %PARAM%

GOTO :END

:SUB_USAGE
   ECHO.
   ECHO %USAGE1% 
   ECHO %USAGE2%  
   ECHO %USAGE3%
   ECHO %USAGE4%
   ECHO.
   EXIT /B

:ABEND

:END
